"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = __importStar(require("@oclif/command"));
const rest_1 = require("@octokit/rest");
const debug_1 = require("../../lib/debug");
const git_1 = require("../../lib/git");
const context_1 = require("../../utils/context");
const pr_release_1 = require("../../utils/pr-release");
const release_1 = require("../../utils/release");
const debug = debug_1.rootDebug(__filename);
class GetCurrentCommitVersion extends command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(GetCurrentCommitVersion);
            // Try to get version from preview/stable release
            // stable release preferred over preview
            // preview release preferred over pr release
            //
            // Note:
            //
            // - PR release should not be possible on same commit as stable/preview
            // anyways
            //
            // - PR release is much more costly to calculate than others
            //
            const c = yield release_1.getCurrentCommit();
            debug('got current commit', c);
            // todo these could have `v` prefix
            if (c.releases.stable) {
                debug('counting stable release as version of this commit');
                return process.stdout.write(c.releases.stable.version);
            }
            if (c.releases.preview) {
                debug('counting preview release as version of this commit');
                return process.stdout.write(c.releases.preview.version);
            }
            // Try to get version from pr release
            debug('commit has no release information, checking for pr-releases');
            const ctx = yield context_1.getLocationContext({
                git: git_1.createGit(),
                octokit: new rest_1.Octokit({
                    auth: process.env.GITHUB_TOKEN,
                }),
            });
            debug('got location context', ctx);
            if (ctx.currentBranch.pr) {
                const version = pr_release_1.getPullRequestReleaseVersionForLocation({
                    packageName: ctx.package.name,
                    prNum: ctx.currentBranch.pr.number,
                    sha: c.sha,
                });
                debug('pr release version for this location context?', { version });
                if (version) {
                    debug('counting pr-release version as version of this commit');
                    return process.stdout.write(version);
                }
            }
            // Give up, with error if specified to
            const giveUpWithError = !flags['optional'];
            debug('giving up', { giveUpWithError });
            if (giveUpWithError) {
                return this.exit(1);
            }
        });
    }
}
exports.GetCurrentCommitVersion = GetCurrentCommitVersion;
GetCurrentCommitVersion.flags = {
    optional: command_1.flags.boolean({
        description: 'Exit 0 if a version for the commit cannot be found',
        default: false,
        char: 'r',
    }),
};
