"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = __importStar(require("@oclif/command"));
const npm_auth_1 = require("../../lib/npm-auth");
const context_1 = require("../../utils/context");
const context_checkers_1 = require("../../utils/context-checkers");
const contrext_guard_1 = require("../../utils/contrext-guard");
const Output = __importStar(require("../../utils/output"));
const pr_release_1 = require("../../utils/pr-release");
const Publish = __importStar(require("../../utils/publish"));
class PR extends command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(PR);
            const context = yield context_1.getContext();
            const report = contrext_guard_1.check({ context })
                .errorUnless(context_checkers_1.npmAuthSetup())
                .errorUnless(branchHasOpenPR())
                // todo only we if can figoure the commits since last pr release
                // .must(haveMeaningfulCommitsInTheSeries())
                .run();
            // if we have preflight errors then we won't be able to calculate the
            // release info. But if doing a dry-run we want to be consistent with
            // other commands that embed report into returned data rather than
            // throwing an error.
            if (report.errors.length && flags['dry-run']) {
                return Output.outputOk('dry_run', {
                    report: report,
                    release: null,
                });
            }
            if (report.errors.length) {
                contrext_guard_1.guard({ context, report, json: flags.json });
            }
            if (flags.json && report.stops.length) {
                Output.didNotPublish({ reasons: report.stops });
                return this.exit(0);
            }
            const versionPrefix = `0.0.0-pr.${context.currentBranch.pr.number}.`;
            const versionBuildNum = pr_release_1.getNextPreReleaseBuildNum(context.package.name, versionPrefix);
            const version = `${versionPrefix}${versionBuildNum}.${context.series.current.sha.slice(0, 7)}`;
            const versionInfo = {
                major: 0,
                minor: 0,
                patch: 0,
                // prettier-ignore
                version: version,
                vprefix: false,
                preRelease: {
                    identifier: 'pr',
                    prNum: context.currentBranch.pr.number,
                    shortSha: context.series.current.sha.slice(0, 7),
                },
            };
            // todo show publish plan in dryrun for other commands too
            const publishPlan = {
                release: {
                    distTag: `pr.${context.currentBranch.pr.number}`,
                    version: versionInfo.version,
                },
                options: {
                    gitTag: 'none',
                    showProgress: !flags.json,
                },
            };
            if (flags['dry-run']) {
                return Output.outputOk('dry_run', {
                    report: report,
                    publishPlan: publishPlan,
                });
            }
            npm_auth_1.setupNPMAuthfileOnCI();
            yield Publish.publish(publishPlan);
            if (flags.json) {
                Output.didPublish({ release: publishPlan.release });
            }
        });
    }
}
exports.PR = PR;
PR.flags = {
    'dry-run': command_1.flags.boolean({
        default: false,
        description: 'output what the next version would be if released now',
        char: 'd',
    }),
    json: command_1.flags.boolean({
        default: false,
        description: 'format output as JSON',
        char: 'j',
    }),
};
//
// Validators
//
function branchHasOpenPR() {
    return {
        code: 'pr_release_without_open_pr',
        summary: 'Pull-Request releases are only supported on branches with _open_ pull-requests',
        run(ctx) {
            return ctx.currentBranch.pr !== null;
        },
    };
}
