"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = __importStar(require("@oclif/command"));
const npm_auth_1 = require("../../lib/npm-auth");
const context_1 = require("../../utils/context");
const context_checkers_1 = require("../../utils/context-checkers");
const contrext_guard_1 = require("../../utils/contrext-guard");
const Output = __importStar(require("../../utils/output"));
const publish_1 = require("../../utils/publish");
const Rel = __importStar(require("../../utils/release"));
class Stable extends command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Stable);
            const context = yield context_1.getContext({
                overrides: {
                    trunk: flags.trunk || null,
                },
            });
            const report = contrext_guard_1.check({ context })
                .errorUnless(context_checkers_1.npmAuthSetup())
                .errorUnless(context_checkers_1.isTrunk())
                .errorUnless(context_checkers_1.branchSynced())
                .stopUnless(notAlreadyStableRelease())
                .stopUnless(haveMeaningfulCommitsInTheSeries())
                .run();
            const maybeRelease = Rel.getNextStable(context.series);
            if (flags['dry-run']) {
                return Output.outputOk('dry_run', {
                    report,
                    release: maybeRelease,
                    commits: context.series.commitsInNextStable,
                });
            }
            if (report.errors.length) {
                contrext_guard_1.guard({ context, report, json: flags.json });
            }
            if (flags.json && report.stops.length) {
                Output.didNotPublish({ reasons: report.stops });
                return this.exit(0);
            }
            const release = maybeRelease; // now validated
            const publishPlan = {
                release: {
                    version: release.version.version,
                    distTag: 'latest',
                    additiomalDistTags: ['next'],
                },
                options: {
                    showProgress: !flags.json,
                    skipNPM: flags['skip-npm'],
                },
            };
            npm_auth_1.setupNPMAuthfileOnCI();
            yield publish_1.publish(publishPlan);
            if (flags.json) {
                Output.didPublish({ release: publishPlan.release });
            }
        });
    }
}
exports.Stable = Stable;
Stable.flags = {
    trunk: command_1.flags.string({
        default: '',
        description: 'State which branch is trunk. Defaults to honuring the "base" branch setting in the GitHub repo settings.',
    }),
    'dry-run': command_1.flags.boolean({
        default: false,
        description: 'output what the next version would be if released now',
        char: 'd',
    }),
    json: command_1.flags.boolean({
        default: false,
        description: 'format output as JSON',
        char: 'j',
    }),
    'skip-npm': command_1.flags.boolean({
        default: false,
        description: 'skip the step of publishing the package to npm',
    }),
};
//
// Validators
//
function haveMeaningfulCommitsInTheSeries() {
    return {
        code: 'series_only_has_meaningless_commits',
        summary: 'A stable release must have at least one semantic commit',
        run(ctx) {
            const release = Rel.getNextStable(ctx.series);
            return release !== 'no_meaningful_change';
            // todo
            // hint:   //             'All commits are either meta or not conforming to conventional commit. No release will be made.',
        },
    };
}
function notAlreadyStableRelease() {
    return {
        code: 'commit_already_has_stable_release',
        summary: 'A stable release requires the commit to have no existing stable release',
        run(ctx) {
            return ctx.series.current.releases.stable === null;
        },
    };
}
