"use strict";
/**
 * Generic extensions to the simple-git library.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CP = __importStar(require("child_process"));
const parse_git_config_1 = __importDefault(require("parse-git-config"));
const parse_github_url_1 = __importDefault(require("parse-github-url"));
const promise_1 = __importDefault(require("simple-git/promise"));
function createGit() {
    return promise_1.default();
}
exports.createGit = createGit;
function parseGitTags(tagsString) {
    if (tagsString === null)
        return [];
    const tags = tagsString
        .trim()
        .split('\n')
        .map(t => t.trim());
    return tags;
}
/**
 * Get tags at the given commit or HEAD by default.
 */
function gitGetTags(git, opts) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const ref = (_b = (_a = opts) === null || _a === void 0 ? void 0 : _a.ref, (_b !== null && _b !== void 0 ? _b : 'HEAD'));
        const tagsString = yield git.tag({ '--points-at': ref });
        const tags = parseGitTags(tagsString);
        return tags;
    });
}
exports.gitGetTags = gitGetTags;
/**
 * Get all tags in the git repo.
 */
function gitGetTagsInRepo(git) {
    return __awaiter(this, void 0, void 0, function* () {
        const tagsString = yield git.raw(['tag']);
        if (tagsString === null)
            return [];
        const tags = tagsString
            .trim()
            .split('\n')
            .map(t => t.trim());
        return tags;
    });
}
exports.gitGetTagsInRepo = gitGetTagsInRepo;
/**
 * Reset a git repository to its last commit, removing staged files, cleaning
 * dirty working directory, etc.
 */
function gitReset(git) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Promise.all([
            git.raw(['clean', '-d', '-x', '-f']),
            gitDeleteAllTagsInRepo(git),
            git.raw(['reset', '--hard']),
        ]);
    });
}
exports.gitReset = gitReset;
/**
 * Reset a git repository to its initial commit
 */
function gitResetToInitialCommit(git) {
    return __awaiter(this, void 0, void 0, function* () {
        yield git.raw(['clean', '-d', '-x', '-f']);
        const trunkBranch = 'master';
        yield git.raw(`checkout ${trunkBranch}`.split(' '));
        yield git
            .raw('rev-list --max-parents=0 HEAD'.split(' '))
            .then(initialCommitSHA => {
            git.raw(['reset', '--hard', initialCommitSHA.trim()]);
        }),
            gitDeleteAllTagsInRepo(git);
    });
}
exports.gitResetToInitialCommit = gitResetToInitialCommit;
/**
 * Get the SHA at the given commit or HEAD by default. By default returns the
 * full SHA.
 */
function gitGetSha(git, opts) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const args = [];
        if (((_a = opts) === null || _a === void 0 ? void 0 : _a.short) === true)
            args.push('--short');
        if (typeof ((_b = opts) === null || _b === void 0 ? void 0 : _b.ref) === 'string') {
            args.push(opts.ref);
        }
        else {
            args.push('HEAD');
        }
        return git.revparse(args);
    });
}
exports.gitGetSha = gitGetSha;
/**
 * Reset not only the working directory but the git repo itself.
 */
function gitInitRepo(git) {
    return __awaiter(this, void 0, void 0, function* () {
        yield git.init();
        yield git.raw(['add', '-A']);
        yield gitCreateEmptyCommit(git, 'initial commit');
    });
}
exports.gitInitRepo = gitInitRepo;
/**
 * Create an empty commit in the repo.
 */
function gitCreateEmptyCommit(git, messge) {
    return __awaiter(this, void 0, void 0, function* () {
        yield git.raw([
            'commit',
            '--allow-empty',
            '--message',
            (messge !== null && messge !== void 0 ? messge : 'Nothing to see here, move along'),
        ]);
    });
}
exports.gitCreateEmptyCommit = gitCreateEmptyCommit;
function createFixCommit(git, msg) {
    return __awaiter(this, void 0, void 0, function* () {
        return gitCreateEmptyCommit(git, `fix: ${(msg !== null && msg !== void 0 ? msg : 'tu tu tu')}`);
    });
}
exports.createFixCommit = createFixCommit;
function createFeatCommit(git, msg) {
    return __awaiter(this, void 0, void 0, function* () {
        return gitCreateEmptyCommit(git, `feat: ${(msg !== null && msg !== void 0 ? msg : 'ti ti ti')}`);
    });
}
exports.createFeatCommit = createFeatCommit;
/**
 * Delete all tags in the repo.
 */
function gitDeleteAllTagsInRepo(git) {
    return __awaiter(this, void 0, void 0, function* () {
        const tags = yield gitGetTagsInRepo(git);
        if (tags.length) {
            yield git.raw(['tag', '-d', ...tags]);
        }
    });
}
exports.gitDeleteAllTagsInRepo = gitDeleteAllTagsInRepo;
/**
 * Check how the local branch is not in sync or is with the remote.
 * Ref: https://stackoverflow.com/questions/3258243/check-if-pull-needed-in-git
 */
function checkSyncStatus(git) {
    return __awaiter(this, void 0, void 0, function* () {
        yield git.remote(['update']);
        const remoteHeads = yield git.raw(['ls-remote', '--heads']);
        const branchSumamry = yield git.branch({});
        const branchOnRemoteRE = new RegExp(`.*refs/heads/${branchSumamry.current}$`, 'm');
        if (remoteHeads.match(branchOnRemoteRE) === null) {
            return 'remote_needs_branch';
        }
        const [local, remote, base] = yield Promise.all([
            git.raw(['rev-parse', '@']).then(sha => sha.trim()),
            git.raw(['rev-parse', '@{u}']).then(sha => sha.trim()),
            git.raw(['merge-base', '@', '@{u}']).then(sha => sha.trim()),
        ]);
        return local === remote
            ? 'synced'
            : local === base
                ? 'needs_pull'
                : remote === base
                    ? 'needs_push'
                    : 'diverged';
    });
}
exports.checkSyncStatus = checkSyncStatus;
/**
 * Extract the github repo name and owner from the git config. If anything goes
 * wrong during extraction a specific error about it will be thrown.
 */
function parseGithubRepoInfoFromGitConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        // Inspiration from how `$ hub pr show` works
        // https://github.com/github/hub/blob/a5fbf29be61a36b86c7f0ff9e9fd21090304c01f/commands/pr.go#L327
        const gitConfig = yield parse_git_config_1.default();
        if (gitConfig === null) {
            throw new Error('Could not parse your git config');
        }
        const gitOrigin = gitConfig['remote "origin"'];
        if (gitOrigin === undefined) {
            throw new Error('Could not find a configured origin in your git config');
        }
        const gitOriginURL = gitOrigin['url'];
        if (gitOriginURL === undefined) {
            throw new Error('Could not find a URL in your remote origin config in your git config');
        }
        const githubRepoURL = parse_github_url_1.default(gitOriginURL);
        if (githubRepoURL === null) {
            throw new Error('Could not parse the URL in your remote origin config in your git config');
        }
        if (githubRepoURL.owner === null) {
            throw new Error('Could not parse out the GitHub owner from the URL in your remote origin config in your git config');
        }
        if (githubRepoURL.name === null) {
            throw new Error('Could not parse out the GitHub repo name from the URL in your remote origin config in your git config');
        }
        return {
            name: githubRepoURL.name,
            owner: githubRepoURL.owner,
        };
    });
}
exports.parseGithubRepoInfoFromGitConfig = parseGithubRepoInfoFromGitConfig;
/**
 * Determin if the current branch is trunk or not. Currently a simple check
 * against if current branch is master or not but TODO in the future will
 * account for checking against the remote Git repo for if the so-called `base`
 * branch of the repo is set to something else than `master`.
 */
function isTrunk(git) {
    return __awaiter(this, void 0, void 0, function* () {
        const branchSumamry = yield git.branch({});
        return branchSumamry.current === 'master';
    });
}
exports.isTrunk = isTrunk;
/**
 * Get the last tag in the current branch that matches the given pattern.
 * Returns null if no tag can be found.
 */
function findTag(git, ops) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        // References about ordering:
        // - https://stackoverflow.com/questions/18659959/git-tag-sorted-in-chronological-order-of-the-date-of-the-commit-pointed-to
        // - https://git-scm.com/docs/git-for-each-ref#_field_names
        // References about by-branch:
        // - https://stackoverflow.com/a/39084124/499537
        const branchSummary = yield git.branch({});
        let tagsByCommits;
        if (ops.since) {
            const logs = yield log(git, { since: (_b = (_a = ops) === null || _a === void 0 ? void 0 : _a.since, (_b !== null && _b !== void 0 ? _b : undefined)) });
            tagsByCommits = logs.map(log => log.tags);
        }
        else {
            // TODO this method flattens tags on same commit to appear as adjacent tags
            // in the list. Seems incidentally technically ok for our current algorithm but dubious...
            const tagsString = yield git.tag({
                // '--sort': 'taggerdate',
                '--merged': branchSummary.current,
            });
            tagsByCommits = parseGitTags(tagsString).map(tag => [tag]);
        }
        let lastTag = null;
        outerloop: for (const tbc of tagsByCommits) {
            for (const tag of tbc) {
                if (ops.matcher(tag)) {
                    lastTag = tag;
                    break outerloop;
                }
            }
        }
        return lastTag;
    });
}
exports.findTag = findTag;
/**
 * Fetch commit log from given ref or else its entirety. The since commit is
 * inclusive. If given, it will be in your result set. Oldest commit comes first.
 */
function log(git, ops) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const gitArgs = ['log', `--format=${gitLogFormat(exports.commitDatums)}`];
        // ~1 makes the range inclusive, since-commit will be in the result set
        if ((_a = ops) === null || _a === void 0 ? void 0 : _a.since)
            gitArgs.push(`${ops.since}~1..head`);
        const rawLog = (_b = (yield git.raw(gitArgs)), (_b !== null && _b !== void 0 ? _b : ''));
        return parseRawLog(rawLog).map(parseLogRefs);
    });
}
exports.log = log;
const logEntrySeparator = '$@<!____LOG____!>@$';
const logEntryValueSepartaor = '$@<!____PROP____!>@$';
exports.commitDatums = [
    { name: 'sha', code: 'H' },
    { name: 'refs', code: 'D' },
    { name: 'message', code: 'B' },
];
const commitDatumNames = exports.commitDatums.map(datum => datum.name);
function gitLogFormat(commitDatums) {
    return (commitDatums.map(part => '%' + part.code).join(logEntryValueSepartaor) +
        logEntrySeparator);
}
exports.gitLogFormat = gitLogFormat;
function parseRawLog(rawLog) {
    const logStrings = rawLog.trim().split(logEntrySeparator);
    logStrings.pop(); // trailing separator
    logStrings.reverse(); // oldest first
    return logStrings.reduce((logs, logString) => {
        logs.push(parseRawLogEntry(logString));
        return logs;
    }, []);
}
exports.parseRawLog = parseRawLog;
function parseRawLogEntry(rawLogEntry) {
    let log = {};
    // propsRemaining and logParts are guaranteed to be the same length
    // TODO should be a zip...
    const propsRemaining = [...commitDatumNames];
    const logParts = rawLogEntry.split(logEntryValueSepartaor);
    while (propsRemaining.length > 0) {
        // @ts-ignore
        log[propsRemaining.shift()] = logParts.shift().trim();
    }
    return log;
}
exports.parseRawLogEntry = parseRawLogEntry;
/**
 * Parse a raw git log into a list of structured commit entries. The given
 * rawLog should be newest commits first, as per how git log returns items (ref:
 * https://stackoverflow.com/questions/18659959/git-tag-sorted-in-chronological-order-of-the-date-of-the-commit-pointed-to#comment95151860_36636526).
 * In turn, the items are here flipped so that oldest commits are first.
 */
function parseLogRefs(_a) {
    var { refs } = _a, rest = __rest(_a, ["refs"]);
    return Object.assign(Object.assign({}, rest), { tags: refs
            .trim()
            .split(', ')
            .map(ref => {
            const result = ref.match(/tag: (.+)/);
            if (!result)
                return null;
            return result[1];
        })
            .filter((tagRef) => typeof tagRef === 'string') });
}
exports.parseLogRefs = parseLogRefs;
/**
 * Lightweight utility to build up a raw log from semi-structured commit input.
 * Useful for creating mock data. Not very safe. Requires tuple members to be in
 * the exact order as the datums in this module. Hence this is for testing, not
 * external consumption.
 */
function serializeLog(values) {
    if (values.length === 0)
        return '';
    return (values.map(v => v.join(logEntryValueSepartaor)).join(logEntrySeparator) +
        logEntrySeparator);
}
exports.serializeLog = serializeLog;
function streamLog(opts) {
    var _a;
    return __asyncGenerator(this, arguments, function* streamLog_1() {
        var e_1, _a;
        const logStream = CP.spawn('git', ['log', `--format=${gitLogFormat(exports.commitDatums)}`, '--no-merges'], { cwd: (_a = opts) === null || _a === void 0 ? void 0 : _a.cwd }).stdout;
        let buffer = '';
        try {
            for (var logStream_1 = __asyncValues(logStream), logStream_1_1; logStream_1_1 = yield __await(logStream_1.next()), !logStream_1_1.done;) {
                const rawLogChunk = logStream_1_1.value;
                buffer += rawLogChunk;
                const rawLogs = buffer.split(logEntrySeparator);
                // 1. No guarantee that we have a clean end of the raw log entry. It could be
                //    short or it could contain some of the next log entry.
                // 2. Safe `!` b/c at a minimum must be [ '' ]
                const maybePartialRawLog = rawLogs.pop();
                buffer = maybePartialRawLog;
                for (const rawLog of rawLogs) {
                    yield yield __await(parseLogRefs(parseRawLogEntry(rawLog)));
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (logStream_1_1 && !logStream_1_1.done && (_a = logStream_1.return)) yield __await(_a.call(logStream_1));
            }
            finally { if (e_1) throw e_1.error; }
        }
        // Flush the buffer. Should always contain the trailing separator since at
        // last log entry.
        const rawLogs = buffer.split(logEntrySeparator);
        rawLogs.pop(); // trailing log separator
        for (const rawLog of rawLogs) {
            yield yield __await(parseLogRefs(parseRawLogEntry(rawLog)));
        }
    });
}
exports.streamLog = streamLog;
