"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("./debug");
const debug = debug_1.rootDebug(__filename);
/**
 * Parse the Github CI Environment. Returns null if parsing fails which should
 * mean it is not a Github CI Environment.
 *
 * @remarks
 *
 * Github docs: https://help.github.com/en/actions/configuring-and-managing-workflows/using-environment-variables#default-environment-variables
 */
function parseGithubCIEnvironment() {
    var _a, _b, _c, _d;
    if (!isGithubCIEnvironment())
        return null;
    let prNum;
    if (process.env.GITHUB_REF) {
        const match = process.env.GITHUB_REF.match(/refs\/pull\/(\d+)\/merge/);
        if (match) {
            debug('found a pr number from github ci environment %s', match[1]);
            prNum = parseInt(match[1], 10);
        }
    }
    const repoPath = process.env.GITHUB_REPOSITORY.split('/');
    const repo = {
        owner: repoPath[0],
        name: repoPath[1],
    };
    let branchName = null;
    if (process.env.GITHUB_HEAD_REF) {
        branchName = process.env.GITHUB_HEAD_REF;
    }
    else if (process.env.GITHUB_REF) {
        branchName = (_b = (_a = process.env.GITHUB_REF.match(/^refs\/heads\/(.+)$/)) === null || _a === void 0 ? void 0 : _a[1], (_b !== null && _b !== void 0 ? _b : null));
    }
    return {
        runId: parseInt(process.env.GITHUB_RUN_ID, 10),
        eventName: process.env
            .GITHUB_EVENT_NAME,
        ref: (_c = process.env.GITHUB_REF, (_c !== null && _c !== void 0 ? _c : null)),
        headRef: (_d = process.env.GITHUB_HEAD_REF, (_d !== null && _d !== void 0 ? _d : null)),
        repository: process.env.GITHUB_REPOSITORY,
        parsed: {
            prNum: prNum,
            repo: repo,
            branchName: branchName,
        },
    };
}
exports.parseGithubCIEnvironment = parseGithubCIEnvironment;
/**
 * Check if the current process appers to be running in a Github CI environment.
 */
function isGithubCIEnvironment() {
    return process.env.GITHUB_RUN_ID !== undefined;
}
exports.isGithubCIEnvironment = isGithubCIEnvironment;
