"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-jetpack"));
const os = __importStar(require("os"));
const path = __importStar(require("path"));
const debug_1 = require("./debug");
const github_ci_environment_1 = require("./github-ci-environment");
const debug = debug_1.rootDebug(__filename);
const TOKEN_ENV_VAR_NAME = 'NPM_TOKEN';
const npmrcFilePath = path.join(os.homedir(), '.npmrc');
/**
 * If in a CI environment and and there is an NPM_TOKEN environment variable
 * then this function will setup an auth  file that permits subsequent package
 * publishing commands.
 */
function setupNPMAuthfileOnCI() {
    if (github_ci_environment_1.isGithubCIEnvironment() && process.env.NPM_TOKEN) {
        const authContent = `//registry.npmjs.org/:_authToken=${process.env[TOKEN_ENV_VAR_NAME]}`;
        debug('writing or appending npm token to %s', npmrcFilePath);
        fs.append(npmrcFilePath, authContent);
    }
}
exports.setupNPMAuthfileOnCI = setupNPMAuthfileOnCI;
/**
 * Read the npmrc file or return null if not found.
 */
function getNpmrcFile() {
    var _a;
    return _a = fs.read(npmrcFilePath), (_a !== null && _a !== void 0 ? _a : null);
}
/**
 * Find out whether NPM auth is setup or not. Setup means dripip upon publishing
 * will have credentials in place to work with the registry. Being setup is
 * satisfied by any of:
 *
 * - A non-empty NPM_TOKEN environment variable set
 * - An npmrc file containing auth
 */
function validateNPMAuthSetup() {
    var _a;
    const npmrc = getNpmrcFile();
    if (npmrc && npmrc.match(/_authToken=.+/)) {
        return { kind: 'pass', reason: 'npmrc_auth' };
    }
    const token = (_a = process.env[TOKEN_ENV_VAR_NAME], (_a !== null && _a !== void 0 ? _a : null));
    if (token) {
        return { kind: 'pass', reason: 'npm_token_env_var' };
    }
    const fail = { kind: 'fail', reasons: [] };
    if (npmrc === null) {
        fail.reasons.push('no_npmrc');
    }
    else {
        fail.reasons.push('npmrc_missing_auth');
    }
    if (token === null) {
        fail.reasons.push('no_env_var');
    }
    else {
        fail.reasons.push('env_var_empty');
    }
    return fail;
}
exports.validateNPMAuthSetup = validateNPMAuthSetup;
