"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-jetpack"));
function getPackageJson() {
    return __awaiter(this, void 0, void 0, function* () {
        const pj = yield read(process.cwd());
        if (!pj) {
            throw new Error('Could not find pacakge.json');
        }
        return pj;
    });
}
exports.getPackageJson = getPackageJson;
/**
 * Read the package.json file.
 */
function read(cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        return fs.readAsync(fs.path(cwd, 'package.json'), 'json');
        // if (!packageJson) {
        //   throw new Error(
        //     `Looked for but could not find a package.json file at ${packageJsonPath}. This file is required for publishing to the npm registry.`
        //   )
        // }
        // if (typeof packageJson !== 'object') {
        //   throw new Error(
        //     `Found a package.json file at ${packageJsonPath} but it appears to be malformed. It did not parse into an object but rather: ${packageJson}`
        //   )
        // }
    });
}
exports.read = read;
/**
 * Read the package.json file synchronously.
 */
function readSync(cwd) {
    return fs.read(fs.path(cwd, 'package.json'), 'json');
}
exports.readSync = readSync;
/**
 * Write the package.json file.
 */
function write(cwd, object) {
    return __awaiter(this, void 0, void 0, function* () {
        return fs.writeAsync(fs.path(cwd, 'package.json'), object);
    });
}
exports.write = write;
/**
 * Update the package.json located at cwd. The given updater function will
 * receive the parsed package contents and whatever is returned will be written
 * to disk.
 */
function update(cwd, updater) {
    return __awaiter(this, void 0, void 0, function* () {
        const packageJson = yield read(cwd);
        if (packageJson) {
            const packageJsonUpdated = yield updater(packageJson);
            yield fs.writeAsync(fs.path(cwd, 'package.json'), packageJsonUpdated);
        }
    });
}
exports.update = update;
function create(cwd) {
    cwd = (cwd !== null && cwd !== void 0 ? cwd : process.cwd());
    return {
        read: read.bind(null, cwd),
        readSync: readSync.bind(null, cwd),
        write: write.bind(null, cwd),
        update: update.bind(null, cwd),
    };
}
exports.create = create;
