"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const proc = __importStar(require("./proc"));
const PJ = __importStar(require("./package-json"));
const fs = __importStar(require("fs-jetpack"));
/**
 * This module abstracts running package manager commands. It is useful when
 * your users might be using npm or yarn but your code needs to stay agnostic to
 * which one.
 */
/**
 * Detect if being run within a yarn or npm script. Ref
 * https://stackoverflow.com/questions/51768743/how-to-detect-that-the-script-is-running-with-npm-or-yarn/51793644#51793644
 */
function detectScriptRunner() {
    var _a, _b;
    if ((_a = process.env.npm_execpath) === null || _a === void 0 ? void 0 : _a.match(/.+npm-cli.js$/))
        return 'npm';
    if ((_b = process.env.npm_execpath) === null || _b === void 0 ? void 0 : _b.match(/.+yarn.js$/))
        return 'yarn';
    return null;
}
/**
 * Publishes the package.
 *
 * About version handling and package.json:
 *
 * In the case of npm the given version will be written to the package.json
 * before publishing and then unwritten afterward.
 *
 * In the case of yarn, package.json is not touched at all.
 *
 * Either way, it should not be noticable to the user of this function.
 */
function publish(manType, input) {
    return __awaiter(this, void 0, void 0, function* () {
        if (manType === 'yarn') {
            const runString = `yarn publish --tag ${input.tag} --no-git-tag-version --new-version ${input.version}`;
            yield proc.run(runString, { require: true });
        }
        else if (manType === 'npm') {
            const pj = PJ.create(process.cwd());
            const pjd = (yield pj.read()); // assume present and valid package.json has been validated already
            yield pj.write(Object.assign(Object.assign({}, pjd), { version: input.version }));
            const runString = `npm publish --tag ${input.tag}`;
            yield proc.run(runString, { require: true });
            yield pj.write(pjd);
        }
        else {
            utils_1.casesHandled(manType);
        }
    });
}
/**
 * Set an npm "dist-tag" to point at a particular version in the npm registry.
 * If the dist-tag does not exist it will be created. If it already exists it
 * will be moved without error. If it already points to the given version,
 * nothing will happen, including namely no error.
 */
function tag(manType, packageName, input) {
    return __awaiter(this, void 0, void 0, function* () {
        const runString = manType === 'npm'
            ? `npm dist-tags add ${packageName}@${input.packageVersion} ${input.tagName}`
            : manType === 'yarn'
                ? `yarn tag add ${packageName}@${input.packageVersion} ${input.tagName}`
                : utils_1.casesHandled(manType);
        try {
            yield proc.run(runString, { require: true });
        }
        catch (e) {
            if (manType === 'yarn' &&
                e.message.match(/error Couldn't add tag./) &&
                e.message.match(/error An unexpected error occurred: ""./)) {
                try {
                    fs.remove('yarn-error.log');
                }
                catch (e) {
                    // silence error if for some reason we cannot clean up the yarn error
                    // log because it is not important enough to handle/tell user about.
                }
                // Silence error here because there is a bug with yarn where tag change command errors out but
                // actually work, ref: https://github.com/yarnpkg/yarn/issues/7823
            }
            else {
                throw e;
            }
        }
    });
}
function create(input) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const packageManagerType = (_a = detectScriptRunner(), (_a !== null && _a !== void 0 ? _a : input.defualt));
        const packageJson = PJ.readSync(process.cwd()); // todo
        return {
            publish: publish.bind(null, packageManagerType),
            tag: tag.bind(null, packageManagerType, packageJson.name),
        };
    });
}
exports.create = create;
