/// <reference types="node" />
import { SpawnOptions } from 'child_process';
export declare type SuccessfulRunResult = {
    command: string;
    stderr: null | string;
    stdout: null | string;
    signal: null | string;
    exitCode: null | number;
    error: null | Error;
};
export declare type RunOptions = SpawnOptions & {
    envAdditions?: Record<string, string | undefined>;
    require?: boolean;
};
/**
 * A wrapper around spawn, easier to use.
 */
export declare function run(commandRaw: string, options?: RunOptions): Promise<SuccessfulRunResult>;
/**
 * Util that binds a command to run making it easy to abstract a parent command.
 *
 * @example
 *
 *    const git = createRunner('git')
 *    const result = await git('status')
 */
export declare const createRunner: (cwd: string) => typeof run;
/**
 * Check if this process was created from the bin of the given project or not.
 * @param packageJsonPath
 */
export declare function isProcessFromProjectBin(packageJsonPath: string): boolean;
/**
 * Log a meaningful semantic error message sans stack track and then crash
 * the program with exit code 1. Parameters are a passthrough to `console.error`.
 */
export declare function fatal(format: string, ...vars: unknown[]): never;
//# sourceMappingURL=proc.d.ts.map