"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
function dump(arg) {
    console.error(util_1.inspect(arg, { depth: 30 }));
}
exports.dump = dump;
/**
 * Indent a string that spans multiple lines.
 */
function indentBlock(size, block) {
    return block
        .split('\n')
        .map(line => range(size)
        .map(exports.constant(' '))
        .join('') + line)
        .join('\n');
}
exports.indentBlock = indentBlock;
exports.indentBlock4 = indentBlock.bind(null, 4);
/**
 * Create a function that will only ever return the given value when called.
 */
exports.constant = (x) => {
    return function () {
        return x;
    };
};
/**
 * Create a range of integers.
 */
const range = (times) => {
    const list = [];
    while (list.length < times) {
        list.push(list.length + 1);
    }
    return list;
};
// type IndexableKeyTypes = string | number | symbol
// type Indexable<T = unknown> = Record<string | number, T>
// type JustIndexableTypes<T> = T extends IndexableKeyTypes ? T : never
// type KeysMatching<Rec, Keys> = NonNullable<
//   {
//     [RecKey in keyof Rec]: Rec[RecKey] extends Keys ? RecKey : never
//   }[keyof Rec]
// >
// export type GroupBy<T extends Indexable, K extends IndexableKeys<T>> = {
//   [KV in JustIndexableTypes<T[K]>]?: Array<T extends Record<K, KV> ? T : never>
// }
// type IndexableKeys<Rec> = KeysMatching<Rec, IndexableKeyTypes>
// export function groupByProp<
//   Obj extends Indexable,
//   KeyName extends IndexableKeys<Obj>
// >(xs: Obj[], keyName: KeyName): GroupBy<Obj, KeyName> {
//   type KeyValue = JustIndexableTypes<Obj[KeyName]>
//   const seed = {} as GroupBy<Obj, KeyName>
//   return xs.reduce((groupings, x) => {
//     const groupName = x[keyName] as KeyValue
//     if (groupings[groupName] === undefined) {
//       groupings[groupName] = []
//     }
//     // We know the group will exist, given above initializer.
//     groupings[groupName]!.push(
//       x as Obj extends Record<KeyName, KeyValue> ? Obj : never
//     )
//     return groupings
//   }, seed)
// }
/**
 * Use this to make assertion at end of if-else chain that all members of a
 * union have been accounted for.
 */
function casesHandled(x) {
    throw new Error(`A case was not handled for value: ${x}`);
}
exports.casesHandled = casesHandled;
/**
 * Determin if the given array or object is empty.
 */
function isEmpty(x) {
    return Array.isArray(x) ? x.length === 0 : Object.keys(x).length > 0;
}
exports.isEmpty = isEmpty;
/**
 * Pause in time for given milliseconds.
 */
function delay(milliseconds) {
    return new Promise(resolve => {
        setTimeout(resolve, 1000);
    });
}
exports.delay = delay;
/**
 * Like Array.findIndex but working backwards from end of array.
 */
function findIndexFromEnd(xs, f) {
    for (let index = xs.length - 1; index > -1; index--) {
        if (f(xs[index]))
            return index;
    }
    return -1;
}
exports.findIndexFromEnd = findIndexFromEnd;
/**
 * Get the last element of an array.
 */
function last(xs) {
    if (xs.length === 0)
        return null;
    return xs[xs.length - 1];
}
exports.last = last;
function numericAscending(n1, n2) {
    if (n1 < n2)
        return -1;
    if (n1 > n2)
        return 1;
    return 0;
}
exports.numericAscending = numericAscending;
