"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const rest_1 = require("@octokit/rest");
const promise_1 = __importDefault(require("simple-git/promise"));
const Git = __importStar(require("../lib/git"));
const github_ci_environment_1 = require("../lib/github-ci-environment");
const PackageJson = __importStar(require("../lib/package-json"));
const Rel = __importStar(require("./release"));
function getContext(opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const git = promise_1.default();
        const octokit = new rest_1.Octokit({
            auth: process.env.GITHUB_TOKEN,
        });
        const locationContext = yield getLocationContext({ git, octokit, opts });
        const series = yield Rel.getCurrentSeries(git);
        return Object.assign({ series }, locationContext);
    });
}
exports.getContext = getContext;
/**
 * Get location-oriented contextual information. Does not consider the release
 * series but things like current branch, repo, pr etc.
 */
function getLocationContext({ git, octokit, opts, }) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const githubCIEnvironment = github_ci_environment_1.parseGithubCIEnvironment();
        // Get repo info
        const repoInfo = (_b = (_a = githubCIEnvironment) === null || _a === void 0 ? void 0 : _a.parsed.repo, (_b !== null && _b !== void 0 ? _b : (yield Git.parseGithubRepoInfoFromGitConfig())));
        // Get which branch is trunk, overridable
        let trunkBranch;
        if ((_d = (_c = opts) === null || _c === void 0 ? void 0 : _c.overrides) === null || _d === void 0 ? void 0 : _d.trunk) {
            trunkBranch = opts.overrides.trunk;
        }
        else {
            const githubRepo = yield octokit.repos.get({
                owner: repoInfo.owner,
                repo: repoInfo.name,
            });
            trunkBranch = githubRepo.data.default_branch;
        }
        // Get the branch
        const branchesSummary = yield git.branch({});
        let currentBranchName;
        if (!branchesSummary.detached) {
            currentBranchName = branchesSummary.current;
        }
        else if (githubCIEnvironment && githubCIEnvironment.parsed.branchName) {
            currentBranchName = githubCIEnvironment.parsed.branchName;
        }
        if (!currentBranchName) {
            throw new Error('Could not get current branch name');
        }
        // Get the pr
        let pr = null;
        if (githubCIEnvironment && githubCIEnvironment.parsed.prNum) {
            pr = {
                number: githubCIEnvironment.parsed.prNum,
            };
        }
        else {
            const maybePR = (yield octokit.pulls.list({
                owner: repoInfo.owner,
                repo: repoInfo.name,
                head: `${repoInfo.owner}:${currentBranchName}`,
                state: 'open',
            })).data[0];
            if (maybePR) {
                pr = {
                    number: maybePR.number,
                };
            }
        }
        // get the branch sync status
        const syncStatus = yield Git.checkSyncStatus(git);
        // get package info
        const packageJson = yield PackageJson.getPackageJson();
        return {
            package: {
                name: packageJson.name,
            },
            githubRepo: {
                owner: repoInfo.owner,
                name: repoInfo.name,
                trunkBranch: trunkBranch,
            },
            currentBranch: {
                name: currentBranchName,
                isTrunk: currentBranchName === trunkBranch,
                pr: pr,
                syncStatus: syncStatus,
            },
        };
    });
}
exports.getLocationContext = getLocationContext;
