declare type Options = {
    /**
     * Should the semver git tag have a "v" prefix.
     */
    gitTagVPrefix?: boolean;
    skipNPM?: boolean;
    /**
     * Should each given dist tag have a corresponding git tag made?
     */
    gitTag?: 'all' | 'just_version' | 'just_dist_tags' | 'none';
    /**
     * Should the publishing process be logged as it progressive?
     */
    showProgress?: boolean;
};
export interface Release {
    /**
     * The version to publish.
     */
    version: string;
    /**
     * The npm dist tag to use for this release.
     */
    distTag: string;
    additiomalDistTags?: string[];
}
export interface PublishPlan {
    release: Release;
    options?: Options;
}
/**
 * Run the publishing process.
 *
 * 1. Change package.json version field to be new version.
 * 2. npm publish --tag next.
 * 3. discard package.json change.
 * 4. git tag {newVer}.
 * 5. git tag next.
 * 6. git push --tags.
 *
 */
export declare function publish(input: PublishPlan): Promise<void>;
export {};
//# sourceMappingURL=publish.d.ts.map