"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ConventionalCommit = __importStar(require("../lib/conventional-commit"));
const Git = __importStar(require("../lib/git"));
const Semver = __importStar(require("../lib/semver"));
/**
 * Get the previous stable and commits since then. If there is no previous
 * stable then commits are counted from start of history.
 */
function getCurrentSeries(git) {
    return __awaiter(this, void 0, void 0, function* () {
        return getLog().then(buildSeries);
    });
}
exports.getCurrentSeries = getCurrentSeries;
/**
 * Like `getLog` but works on the given data. Useful for unit testing with mock
 * log data.
 */
function fromLogs(entries) {
    const commits = [];
    let previousStableCommit = null;
    for (const log of entries) {
        if (log.tags.find(isStableTag)) {
            previousStableCommit = log;
            break;
        }
        commits.push(log);
    }
    return buildSeries([previousStableCommit, commits]);
}
exports.fromLogs = fromLogs;
/**
 * Get the latest stable and all subsequent commits. If no stable then all
 * commits (edge-case, new project) since-inclusive initial commit. The returned
 * tuple contains the  stable commit partitioned from the subsequent commits.
 * The stable commit may be null.
 */
function getLog() {
    var e_1, _a;
    return __awaiter(this, void 0, void 0, function* () {
        const commits = [];
        let previousStableCommit = null;
        try {
            for (var _b = __asyncValues(Git.streamLog()), _c; _c = yield _b.next(), !_c.done;) {
                const log = _c.value;
                if (log.tags.find(isStableTag)) {
                    previousStableCommit = log;
                    break;
                }
                commits.push(log);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return [previousStableCommit, commits];
    });
}
/**
 * Get the current commit.
 */
function getCurrentCommit() {
    var e_2, _a;
    return __awaiter(this, void 0, void 0, function* () {
        let currLog;
        try {
            for (var _b = __asyncValues(Git.streamLog()), _c; _c = yield _b.next(), !_c.done;) {
                const log = _c.value;
                currLog = log;
                break;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        if (!currLog) {
            throw new Error('There are no commits in this repo');
        }
        return logEntryToCommit(currLog);
    });
}
exports.getCurrentCommit = getCurrentCommit;
/**
 * Transform a log entry into a commit.
 */
function logEntryToCommit(log) {
    var _a, _b;
    const parsedMessage = ConventionalCommit.parse(log.message);
    const previewTag = (_a = log.tags.find(isPreviewTag), (_a !== null && _a !== void 0 ? _a : null));
    const stableTag = (_b = log.tags.find(isStableTag), (_b !== null && _b !== void 0 ? _b : null));
    const otherTags = log.tags.filter(isUnknownTag);
    return {
        sha: log.sha,
        message: {
            parsed: parsedMessage,
            raw: log.message,
        },
        releases: {
            stable: stableTag ? Semver.parse(stableTag) : null,
            preview: previewTag ? Semver.parsePreview(previewTag) : null,
        },
        nonReleaseTags: otherTags,
    };
}
/**
 * Build structured series data from a raw series log.
 */
function buildSeries([previousStableLogEntry, commitsSincePrevStable,]) {
    var _a, _b, _c, _d;
    if (previousStableLogEntry === null && commitsSincePrevStable.length === 0) {
        throw new Error(`Cannot build release series with given data. There is no previous stable release and no commits since. This would indicate an unexpected error or working with a git repo that has zero commits. The latter should be guarded by upstream checks. Therefore this is bad. There must be a bug.`);
    }
    let hasBreakingChange = false;
    const commitsInNextStable = commitsSincePrevStable.map(c => {
        var _a, _b, _c;
        const parsedMessage = ConventionalCommit.parse(c.message);
        if ((_a = parsedMessage) === null || _a === void 0 ? void 0 : _a.breakingChange) {
            hasBreakingChange = true;
        }
        return {
            message: {
                raw: c.message,
                parsed: parsedMessage,
            },
            sha: c.sha,
            nonReleaseTags: c.tags.filter(isUnknownTag),
            releases: {
                stable: Semver.parse((_b = c.tags.find(isStableTag), (_b !== null && _b !== void 0 ? _b : ''))),
                preview: Semver.parsePreview((_c = c.tags.find(isPreviewTag), (_c !== null && _c !== void 0 ? _c : ''))),
            },
        };
    });
    const previousPreviewIndex = commitsInNextStable.findIndex(c => c.releases.preview !== null);
    const previousPreview = previousPreviewIndex === -1
        ? null
        : commitsInNextStable[previousPreviewIndex];
    const commitsInNextPreview = previousPreviewIndex === -1
        ? commitsInNextStable
        : commitsInNextStable.slice(0, previousPreviewIndex);
    const previousStable = previousStableLogEntry === null
        ? null
        : {
            sha: previousStableLogEntry.sha,
            message: {
                raw: previousStableLogEntry.message,
                parsed: ConventionalCommit.parse(previousStableLogEntry.message),
            },
            nonReleaseTags: previousStableLogEntry.tags.filter(isUnknownTag),
            releases: {
                stable: Semver.parse(previousStableLogEntry.tags.find(isStableTag)),
                preview: Semver.parsePreview((_a = previousStableLogEntry.tags.find(isPreviewTag), (_a !== null && _a !== void 0 ? _a : ''))),
            },
        };
    const isInitialDevelopment = (_c = (_b = previousStable) === null || _b === void 0 ? void 0 : _b.releases.stable.major, (_c !== null && _c !== void 0 ? _c : 0)) < 1;
    return {
        isInitialDevelopment,
        hasBreakingChange,
        previousStable: previousStable,
        previousPreview,
        commitsInNextStable,
        commitsInNextPreview,
        // If there are no commits since stable and no stable that means we're on a
        // repo with no commit. This edge-case is ignored. It is assumed that it
        // will be validated for before calling this function.
        current: (_d = commitsInNextStable[0], (_d !== null && _d !== void 0 ? _d : previousStable)),
    };
}
exports.buildSeries = buildSeries;
function isUnknownTag(tag) {
    return Semver.parse(tag) === null;
}
function isStableTag(tag) {
    const v = Semver.parse(tag);
    if (v === null)
        return false;
    return Semver.isStable(v);
}
function isPreviewTag(tag) {
    const v = Semver.parse(tag);
    if (v === null)
        return false;
    return Semver.isPreview(v);
}
function isNoReleaseReason(x) {
    return x === 'no_meaningful_change' || x === 'empty_series';
}
exports.isNoReleaseReason = isNoReleaseReason;
/**
 * Get the next stable from a given series. If a release cannot be made for the
 * given series then a code indicating why will be returned.
 */
function getNextStable(series) {
    if (series.commitsInNextStable.length === 0) {
        return 'empty_series';
    }
    const bumpType = ConventionalCommit.calcBumpType(series.isInitialDevelopment, series.commitsInNextStable.map(c => c.message.raw));
    if (bumpType === null)
        return 'no_meaningful_change';
    const version = Semver.incStable(bumpType, series.previousStable === null
        ? Semver.zeroVer
        : series.previousStable.releases.stable);
    return { bumpType, version };
}
exports.getNextStable = getNextStable;
/**
 * Calculate the preview release for the given series.
 */
function getNextPreview(series) {
    var _a, _b, _c, _d;
    if (series.commitsInNextStable.length === 0) {
        return 'empty_series';
    }
    // todo test this case, it was fixed without regression test being added
    const bumpTypeContributionFromCommitsInNextPreview = ConventionalCommit.calcBumpType(series.isInitialDevelopment, series.commitsInNextPreview.map(c => c.message.raw));
    if (bumpTypeContributionFromCommitsInNextPreview === null)
        return 'no_meaningful_change';
    const bumpType = ConventionalCommit.calcBumpType(series.isInitialDevelopment, series.commitsInNextStable.map(c => c.message.raw));
    // Stable is superset of preview so based on guard above there MUST be a
    // bumpType here.
    if (!bumpType) {
        throw new Error('next preview in series has bump type but not series as a whole. You should not be seeing this. This should be impossible.');
    }
    const nextStable = Semver.incStable(bumpType, (_b = (_a = series.previousStable) === null || _a === void 0 ? void 0 : _a.releases.stable, (_b !== null && _b !== void 0 ? _b : Semver.zeroVer)));
    const version = Semver.stableToPreview(nextStable, 'next', (_d = (_c = series.previousPreview) === null || _c === void 0 ? void 0 : _c.releases.preview.preRelease.buildNum, (_d !== null && _d !== void 0 ? _d : Semver.zeroBuildNum)) + 1);
    return { bumpType, version };
}
exports.getNextPreview = getNextPreview;
//
// Private helpers
//
// async function findLatestStable(git: Git.Simple): Promise<null | string> {
//   return Git.findTag(git, { matcher: isStableTag })
// }
// const zeroReleases = {
//   stable: null,
//   preview: null,
// }
/**
 * Get the releases at the given commit.
 */
// export async function getReleasesAtCommit(
//   sha: string
// ): Promise<CommitReleases> {
//   const git = createGit()
//   const tags = await Git.gitGetTags(git, { ref: sha })
//   if (isEmpty(tags)) return zeroReleases
//   const parsedTags = groupByProp(tags.map(parseTag), 'type')
//   const stableTags = parsedTags.stable_release ?? []
//   const previewtags = parsedTags.pre_release ?? []
//   const invariantViolations = []
//   if (stableTags.length > 1) {
//     invariantViolations.push(
//       `Multiple stable releases found but there should only be 0 or 1: ${stableTags}`
//     )
//   }
//   if (previewtags.length > 1) {
//     invariantViolations.push(
//       `Multiple preview releases found but there should only be 0 or 1: ${stableTags}`
//     )
//   }
//   if (invariantViolations.length > 0) {
//     throw new Error(
//       `While getting the pre-existing releases at commit ${sha} the following invariant violations were found:\n\n${invariantViolations
//         .map(x => `    - ${x}`)
//         .join('\n')}`
//     )
//   }
//   return {
//     stable: stableTags[0]
//       ? {
//           type: 'stable',
//           version: stableTags[0].value,
//           sha,
//         }
//       : null,
//     preview: previewtags[0]
//       ? {
//           type: 'preview',
//           version: previewtags[0].value,
//           sha,
//           buildNum: previewtags[0].value.prerelease[1] as number,
//         }
//       : null,
//   }
// }
// export type ParsedTag =
//   | { type: 'unknown'; value: string }
//   | { type: 'stable_release'; value: SemVer.SemVer }
//   | { type: 'pre_release'; value: SemVer.SemVer }
// export function parseTag(rawTag: string): ParsedTag {
//   const semverParseResult = SemVer.parse(rawTag)
//   if (semverParseResult !== null) {
//     if (semverParseResult.prerelease.length > 0) {
//       return { type: 'pre_release', value: semverParseResult } as const
//     } else {
//       return { type: 'stable_release', value: semverParseResult } as const
//     }
//   } else {
//     return { type: 'unknown', value: rawTag } as const
//   }
// }
